
#include <windows.h>
#include <stdio.h>
#include "Timer.h"


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32


HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "My Application"; 

// the rest of the stuff
//......................

BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
	WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}



LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}


#define USR_TIMER_ELAPSED  (WM_USER+1)

BOOL CALLBACK TimerDlg(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
VOID CALLBACK timeProc(UINT nTimerId, UINT msg, DWORD dwUser, DWORD dwParam1, DWORD dwParam2);


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   switch( uMsg )
   {
      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST:
                        DialogBox(hInst, "TimerDlg", hWnd, TimerDlg);
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;

      case WM_DESTROY :
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );               
}


BOOL CALLBACK TimerDlg(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
   static UINT nTimerId;
   static UINT nMs;
   static UINT nEvents;  // number of timer events.

   switch (uMsg)
    {
        // init dialog
        //.............
        
        case WM_INITDIALOG:
               {
                  TIMECAPS tc;

                  nTimerId = 0;
                  nEvents  = 0;
                  nMs      = 500;

                  timeGetDevCaps(&tc, sizeof(TIMECAPS) );

                  if (nMs < tc.wPeriodMin)
                      nMs = tc.wPeriodMin;
                  else if (nMs > tc.wPeriodMax)
                      nMs = tc.wPeriodMax;

                  timeBeginPeriod(nMs);

                  nTimerId = timeSetEvent(nMs, nMs, timeProc, (DWORD)hDlg,
                                          TIME_PERIODIC);

                  if (!nTimerId)
                  {                                            
                      timeEndPeriod(nMs);
                      MessageBox(hDlg, "Error setting up timer...", NULL, MB_OK);
                      EndDialog(hDlg, FALSE);
                      return(TRUE);
                  }

               }
               return(TRUE);

        case USR_TIMER_ELAPSED :
               nEvents++;
               SetDlgItemInt(hDlg, IDC_TIMER_EC, nEvents, FALSE);
               break;

        // commands
        //.........

        case WM_COMMAND:
               switch ( LOWORD(wParam) )
               {
                  // close down
                  //...........

                  case WM_CLOSE:
                  case IDOK:
                  case IDCANCEL:
                         {
                            // Shut down timer
                            //................

                            timeKillEvent(nTimerId);
                            timeEndPeriod(nMs);
                            
                            EndDialog(hDlg, TRUE);
                         }
                         break;
               }
               break;

    }

    return (FALSE);
}


VOID CALLBACK timeProc(UINT nTimerId, UINT msg, DWORD dwUser, DWORD dwParam1, DWORD dwParam2)
{
   PostMessage((HWND)dwUser, USR_TIMER_ELAPSED, 0, 0);
}






